part of tecfy_corev2_client_package;

class FileMultiProperty extends StatefulWidget {
  final List<String>? filesList;

  /// label string
  final String? label;
  final String? description;

  /// is required
  final bool isRequired;

  /// field id
  final String? fileId;

  /// field id
  final int maxSize;

  // final int maxHeight;
  final double? aspectRatioMax;
  final double? aspectRatioMin;

  /// on changed
  final void Function(dynamic)? onChanged;

  FileMultiProperty(
      {this.filesList,
      this.label,
      this.description,
      this.maxSize = 200,
      // this.maxHeight = 200,
      this.aspectRatioMax,
      this.aspectRatioMin,
      this.onChanged,
      this.fileId,
      this.isRequired = false}); //: assert(fileId != null );
  @override
  _FileMultiPropertyState createState() => _FileMultiPropertyState();
}

class _FileMultiPropertyState extends State<FileMultiProperty> {
  FilePickerResult? _selectedPhotos;

  _onChoosePhoto() async {
    var maxWidth = widget.maxSize;
    // var maxHeight = widget.maxHeight;
    if (maxWidth == 0) maxWidth = 200;
    // if (maxHeight == 0) maxHeight = 200;
    _selectedPhotos = await FilePicker.platform.pickFiles(allowMultiple: true);
    if (_selectedPhotos != null) {
      AppNavigator.showLoading(context);
      List<String> photosUrlList = [];
      for (PlatformFile photo in _selectedPhotos!.files) {
        var result = await TecfyBasicApp.apiService.httpPostFile(
            'general/storage/file/fullUrl/admin', photo.bytes!, photo.name);
        widget.filesList!.add(result.data!);
        photosUrlList.add(result.data!);
      }
      widget.onChanged?.call(widget.filesList);
      print(photosUrlList);
      Navigator.pop(context);
      setState(() {});
    }
  }

  _onChoosePhotoFromFileManager() async {
    WidgetsBinding.instance.addPostFrameCallback((timeStamp) async {
      List<FileModel>? result = await AppNavigator.showAppDialogEx(context,
          () => FileManagerPage(isPopup: true, isMultipleChoices: true),
          size: DialogBoxSizes.large);
      if (result != null) {
        print(result);
        setState(() {
          widget.onChanged?.call(result.map((e) => e.url).toList());
        });
      }
    });
  }

  @override
  Widget build(BuildContext context) {
    return Tooltip(
      message: widget.description ?? '',
      child: Column(
        children: [
          DropdownButtonHideUnderline(
            child: DropdownButton2(
                items: [
                  DropdownMenuItem(
                      value: 'upload',
                      onTap: _onChoosePhoto,
                      child: Text('Upload')),
                  DropdownMenuItem(
                    value: 'upload',
                    onTap: _onChoosePhotoFromFileManager,
                    child: Text('Select from File Manager',
                        overflow: TextOverflow.ellipsis, maxLines: 1),
                  ),
                ],
                onChanged: (value) {},
                customButton: Row(
                  children: [
                    Icon(Icons.photo_library_rounded),
                    Text('إضافة صور'),
                  ],
                )),
          ),
          SizedBox(
            height: 15,
          ),
          if (widget.filesList != null)
            SingleChildScrollView(
              scrollDirection: Axis.horizontal,
              child: Row(
                  children: widget.filesList!
                      .map((e) => Padding(
                            padding: EdgeInsets.only(left: 10),
                            child: Container(
                              width: 100,
                              height: 100,
                              decoration: BoxDecoration(
                                  border:
                                      Border.all(color: Colors.grey, width: 2),
                                  borderRadius: BorderRadius.circular(5)),
                              child: Stack(
                                alignment: Alignment.bottomLeft,
                                children: <Widget>[
                                  Align(
                                    alignment: Alignment.center,
                                    child: Image.network(e,
                                        width: 100, fit: BoxFit.fitWidth),
                                  ),
                                  IconButton(
                                      onPressed: () {
                                        setState(() {
                                          widget.filesList?.remove(e);
                                          widget.onChanged
                                              ?.call(widget.filesList);
                                        });
                                      },
                                      icon: Icon(
                                        Icons.delete_forever_rounded,
                                        color: Colors.red,
                                        size: 22,
                                      ))
                                ],
                              ),
                            ),
                          ))
                      .toList()),
            ),
        ],
      ),
    );
  }
}
