part of tecfy_corev2_client_package;

class LocationPropertyComponent extends StatefulWidget {
  /// ini value
  final Map<String, dynamic>? initialValue;

  /// label string
  final String? label;
  final String? description;

  /// is requried
  final bool isRequired;

  /// is read only
  final bool readOnly;

  /// on changed
  final void Function(Map<String, dynamic>)? onChanged;

  LocationPropertyComponent(
      {this.initialValue,
      this.label,
      this.description,
      this.isRequired = false,
      this.onChanged,
      this.readOnly = false});

  @override
  _LocationPropertyComponentState createState() =>
      _LocationPropertyComponentState();
}

class _LocationPropertyComponentState extends State<LocationPropertyComponent> {
  LatLng? _point;
  Map<String, dynamic>? _currentValue;

  @override
  initState() {
    if (kDebugMode)
    _currentValue = widget.initialValue;
    if (widget.initialValue == null || widget.initialValue!.isEmpty) {
      _point = null;
    } else {
      _currentValue = widget.initialValue;
      // var p = widget.initalValue!.split(',');
      // if (p.any((element) => element.contains(':'))) {
      //   p.forEach((element) {
      //     var doubleRE = RegExp(r"-?(?:\d*\.)?\d+(?:[eE][+-]?\d+)?");
      //     p[p.indexOf(element)] = doubleRE.stringMatch(element)!;
      //   });
      // }
      // print("------ $p -------");
      // if (p.length >= 1) {
      _point = LatLng(BaseModel.getDoubleEx(_currentValue!['lat'])!,
          BaseModel.getDoubleEx(_currentValue!['lng'])!);
      // }
    }
    if (_point == null) {
      // get my location
      WidgetsBinding.instance.addPostFrameCallback((timeStamp) async {
        var locationData = await Location().getLocation();
        _point =
            LatLng(locationData.latitude ?? 0, locationData.longitude ?? 0);
      });
    }
    super.initState();
  }

  _onChanged() {
    // var r = '${_point?.latitude ?? 0},${_point?.longitude ?? 0}';
    // if (kDebugMode)
    // print('----------------------------- Selected Location is $r');
    _currentValue =
        Map.of({'lat': _point?.latitude ?? 0, 'lng': _point?.longitude ?? 0});
    setState(() {});
    widget.onChanged?.call(_currentValue!);
  }

  @override
  Widget build(BuildContext context) {
    return MaterialButton(
        padding: EdgeInsets.all(0),
        onPressed: () async {
          var result = await showDialog(
              context: context,
              builder: (c) =>
                  LocationSelectPage(_point ?? LatLng(31.274750, 32.301204)));
          if (result != null && result is LatLng) {
            _point = result;
            _onChanged();
          }
        },
        child: Column(
          crossAxisAlignment: CrossAxisAlignment.start,
          children: <Widget>[
            SizedBox(height: 10),
            Tooltip(
                message: widget.description ?? '',
                child: TecfyText(
                  '${widget.label ?? ''}${widget.isRequired ? '*' : ''}',
                  style: ThemeService.labelStyle,
                )),
            TecfyText(_currentValue == null || _currentValue!.isEmpty
                ? '[Click To Select]'
                : _currentValue
                    .toString()
                    .replaceAll('lat:', '')
                    .replaceAll('lng:', '')
                    .replaceAll('{', '')
                    .replaceAll('}', '')),
            SizedBox(height: 10),
            Container(height: 1, color: ThemeService.textSubColor)
          ],
        ));
  }
}

class LocationSelectPage extends StatefulWidget {
  final LatLng location;
  LocationSelectPage(this.location);
  @override
  _LocationSelectPageState createState() => _LocationSelectPageState();
}

class _LocationSelectPageState extends State<LocationSelectPage> {
  // String _mapStyle;
  LatLng? _point;
  GlobalKey _key = GlobalKey();
  // GoogleMapController _controller;
  @override
  initState() {
    _point = widget.location;
    super.initState();
  }

  @override
  Widget build(BuildContext context) {
    return Container(
        width: MediaQuery.of(context).size.width,
        height: MediaQuery.of(context).size.height,
        child: Column(
          children: [
            Expanded(
                flex: 10,
                child: GoogleMap(
                  zoomControlsEnabled: true,
                  // onMapCreated: (c) => _controller = c,
                  key: _key,
                  initialCameraPosition: CameraPosition(
                      zoom: 10,
                      target: _point == null
                          ? LatLng(31.274750, 32.301204)
                          : _point!),
                  // initialPosition: point ?? GeoCoord(0, 0),
                  mapType: MapType.hybrid,
                  indoorViewEnabled: true,
                  myLocationButtonEnabled: true,
                  myLocationEnabled: true,
                  markers: _point == null
                      ? {}
                      : {
                          Marker(
                              position: _point!,
                              markerId: MarkerId("marker2"),
                              icon: BitmapDescriptor.defaultMarker)
                        },

                  onTap: (coord) {
                    setState(() {
                      _point = coord;
                    });
                    // _controller.getLatLng(screenCoordinate)
                    // GoogleMap.of(_key).clearMarkers();
                    // GoogleMap.of(_key).addMarker(Marker(point));
                  },
                )),
            Expanded(
                flex: 1,
                child: Container(
                    color: Colors.white,
                    child: Row(children: [
                      TextButton(
                          child: TecfyText('OK'),
                          onPressed: () {
                            Navigator.pop(context, _point);
                          }),
                      TextButton(
                          child: TecfyText('Cancel'),
                          onPressed: () {
                            Navigator.pop(context);
                          }),
                    ])))
          ],
        ));
  }
}
