part of tecfy_corev2_client_package;

class FileManagerPage extends StatefulWidget {
  final bool isPopup;
  final bool isMultipleChoices;

  const FileManagerPage(
      {Key? key, this.isPopup = false, this.isMultipleChoices = false})
      : super(key: key);

  @override
  State<FileManagerPage> createState() => _FileManagerPageState();
}

class _FileManagerPageState extends State<FileManagerPage> {
  List<FileModel>? _myData;

  Future<List<FileModel>?> getFilesData(String? parentId) async {
    var response = await ApiService().httpGetDynamic(
        '${ApiService.apiURL}admin/setup/file/tree?parentId=${parentId ?? ''}&ski'
        'p=0'
        '&count=0');
    return List<FileModel>.from(
        response.data.map((e) => FileModel.fromJson(e)));
  }

  Future<String?> uploadFile(
      String? folderId, Uint8List? pickedFile, String? pickedFileName) async {
    bool isImage = true;
    if (pickedFileName != null) {
      if (pickedFileName.split('.').last == 'pdf' ||
          pickedFileName.split('.').last == 'xlsx') isImage = false;
    }
    var response = await ApiService().httpPostFile(
        '${ApiService.apiURL}general/storage/folder/file/admin',
        pickedFile!,
        pickedFileName!,
        {
          'type': 'File',
          if (folderId != null) 'parentId': folderId,
        },
        {},
        isImage);
    return response.data;
  }

  Future<String?> createFolder(String? parentId, String? folderName) async {
    var response = await ApiService()
        .httpPostEx('${ApiService.apiURL}general/storage/folder/file/admin', {
      'type': 'Folder',
      'path': folderName,
      if (parentId != null) 'parentId': parentId,
      'name': folderName
    });
    return response.data;
  }

  Future<bool> deleteIds(List<String>? deletedIds) async {
    var response = await ApiService()
        .httpPut('${ApiService.apiURL}admin/setup/file/tree/delete', {
      'ids': deletedIds,
    });
    print(response.data);
    return response.success;
  }

  @override
  void initState() {
    init();

    super.initState();
  }

  init() async {
    _myData = await getFilesData(null);
    setState(() {});
  }

  late List _selectedFiles;

  @override
  Widget build(BuildContext context) {
    if (widget.isPopup) {
      return _myData != null
          ? Column(children: [
              if (widget.isMultipleChoices)
                ElevatedButton(
                    child: Text(AppLocale.translate('Select')),
                    onPressed: () {
                      if (widget.isMultipleChoices)
                        AppNavigator.pop(context, _selectedFiles);
                    }),
              Expanded(
                  child: SimpleFileManager(
                filesList: _myData!,
                isWithUploadAndDownloadButtons: false,
                dropdownItems: [
                  DropdownMenuItem(
                    child: Row(
                      children: <Widget>[
                        Icon(
                          Icons.check_box,
                          color: Colors.white,
                        ),
                        Text(
                          'Select',
                          style: TextStyle(color: Colors.white),
                        )
                      ],
                    ),
                    value: 'select',
                  ),
                ],
                onItemSelected: (List<FileModel>? model) {
                  if (model?.isNotEmpty ?? false) {
                    if (widget.isMultipleChoices) {
                      _selectedFiles = model!;
                    } else {
                      AppNavigator.pop(context, model!.first);
                    }
                  }
                },
                uploadButtonText:
                    LocalConfigService.isArabic ? 'رفع ملف' : 'Upload File',
                createFolderText:
                    LocalConfigService.isArabic ? 'إنشاء ملف' : 'Create Folder',
                // allowedExtensionsToPick: [],
                onUpload:
                    (String? parentId, pickedFiles, pickedFileNames) async {
                  if (pickedFiles != null) {
                    List<String> urlList = [];
                    for (int x = 0; x < pickedFiles.length; x++) {
                      urlList.add(await uploadFile(
                              parentId, pickedFiles[x], pickedFileNames?[x]) ??
                          '');
                    }
                    return urlList;
                  } else {
                    return null;
                  }
                },
                onCreateFolderClicked: (String? parentID, folderName) async {
                  print(parentID);
                  print(folderName);
                  return await createFolder(parentID, folderName);
                },
                onBack: (String? value) async {
                  return await getFilesData(value);
                },
                onFolderClicked: (value) async {
                  return await getFilesData(value!.id);
                },
              )),
            ])
          : Center(
              child: CircularProgressIndicator.adaptive(),
            );
    }
    return BasePage(context, 'الملفات', 'files',
        isMobile: LayoutService.isMobile(context),
        child: _myData != null
            ? SimpleFileManager(
                filesList: _myData!,
                uploadButtonText:
                    LocalConfigService.isArabic ? 'رفع ملف' : 'Upload File',
                createFolderText:
                    LocalConfigService.isArabic ? 'إنشاء ملف' : 'Create Folder',
                // allowedExtensionsToPick: [],
                onUpload:
                    (String? parentId, pickedFiles, pickedFileNames) async {
                  if (pickedFiles != null) {
                    List<String> urlList = [];
                    for (int x = 0; x < pickedFiles.length; x++) {
                      urlList.add(await uploadFile(
                              parentId, pickedFiles[x], pickedFileNames?[x]) ??
                          '');
                    }
                    return urlList;
                  } else {
                    return null;
                  }
                },
                onCreateFolderClicked: (String? parentID, folderName) async {
                  print(parentID);
                  print(folderName);
                  return await createFolder(parentID, folderName);
                },
                onBack: (String? value) async {
                  return await getFilesData(value);
                },
                onFolderClicked: (value) async {
                  return await getFilesData(value!.id);
                },
                onDeleteClicked: (List<String> deletedList) async {
                  return await deleteIds(deletedList);
                },
              )
            : Center(
          child: CircularProgressIndicator.adaptive(),
        ));
  }
}
