part of tecfy_corev2_client_package;

class EntityFilterPage extends StatefulWidget {
  /// final String entityName;
  final List<Property> properties;

  /// default filters
  final List<EntityFilter>? filters;

  /// return the filters list after update
  EntityFilterPage(this.properties, [this.filters]);
  @override
  _EntityFilterPageState createState() => _EntityFilterPageState();
}

class _EntityFilterPageState
    extends TecfyStateView<EntityFilterPage, EntityFilterController> {
  late List<EntityFilter> filters;
  @override
  void initState() {
    _controller = EntityFilterController(this, widget.properties);
    widget.filters?.forEach((element) {
      element.property?.readOnly = false;
      element.property?.isRequired = false;
    });
    filters = widget.filters ?? [];
    super.initState();
    _init();
  }

  Future _init() async {
    for (var prop in widget.properties) {
      prop.readOnly = false;
      prop.isRequired = false;
      if (prop.propertyType == PropertyTypes.Date &&
          (prop as PropertyDate).isTodayByDefault &&
          !filters.any((e) => e.propertyName == prop.dbName)) {
        var d = Map<String, dynamic>();
        if (await prop.getDefaultValue(d)) {
          if (!filters.any((e) => e.propertyName == prop.dbName)) {
            filters.add(EntityFilter(
                operatorId: 'E',
                property: prop,
                propertyName: prop.dbName,
                value: d[prop.dbName]));
          }
        }
      }
    }
    if (filters.length == 0) filters.add(EntityFilter());
    await Future.delayed(Duration(milliseconds: 100));
    setState(() {});
  }

  _onCancel() {
    Navigator.pop(context, null);
  }

  _onFilter() {
    Navigator.pop(context, filters);
  }

  _onAdd() {
    if (filters.any((e) => e.propertyName == null)) return;
    setState(() {
      filters.add(EntityFilter());
    });
  }

  @override
  Widget build(BuildContext context) {
    return Center(
      child: Container(
        constraints: BoxConstraints(
            minWidth: 300, minHeight: 300, maxHeight: 600, maxWidth: 500),
        child: Material(
          // appBar: MainAppBar(context: context,title: '${_controller.entityName} (${AppLocale.translate(context, 'Filter')})', actions: <Widget>[
          // IconButton(icon:Icon(Icons.close),onPressed: onCancel)
          // ],),
          child: DefaultTabController(
            length: 2,
            child: Column(children: [
              TabBar(labelColor: ThemeService.mainColor, tabs: [
                Tab(child: TecfyText('Standard')),
                Tab(child: TecfyText('Advance'))
              ]),
              Expanded(
                child: TabBarView(children: [
                  // standard
                  ListView(
                      padding: EdgeInsets.symmetric(horizontal: 10),
                      children: _controller.properties
                          .map((prop) => _getBasicFilterRow(prop))
                          .toList()),
                  // advance
                  ListView(
                    children: filters
                        .map((filter) => _getAdvanceFilterCard(filter))
                        .toList()
                      ..add(TextButton.icon(
                          icon: Icon(Icons.add),
                          onPressed: _onAdd,
                          label: TecfyText(AppLocale.translate('Add Row')))),
                  ),
                ]),
              ),
              Row(mainAxisAlignment: MainAxisAlignment.spaceEvenly, children: [
                TextButton.icon(
                    icon: Icon(Icons.search),
                    onPressed: _onFilter,
                    label: TecfyText(AppLocale.translate('OK'))),
                TextButton.icon(
                    icon: Icon(Icons.close),
                    onPressed: _onCancel,
                    label: TecfyText(AppLocale.translate('Close')))
              ])
            ]),
          ),
        ),
      ),
    );
  }

  Widget _getBasicFilterRow(Property prop) {
    dynamic val;
    try {
      val = filters.firstWhere((f) => f.propertyName == prop.dbName).value;
    } catch (err) {}
    return PorpertyFilterComponent(
      onChange: (v) {
        filters.removeWhere((f) => f.propertyName == prop.dbName);
        if (v != null && v.toString().length > 0) {
          filters.add(EntityFilter(
              property: prop,
              propertyName: prop.dbName,
              operatorId: 'E',
              value: v));
        }
      },
      property: prop,
      value: val,
    );
  }

  Widget _getAdvanceFilterCard(EntityFilter filter) {
    return Card(
      margin: EdgeInsets.all(10),
      borderOnForeground: true,
      color: Colors.lightBlue[100],
      child: Column(
        children: <Widget>[
          Align(
              alignment: Alignment.topRight,
              child: IconButton(
                  icon: Icon(
                    Icons.close,
                    color: Colors.red,
                  ),
                  onPressed: () {
                    filters.remove(filter);
                    _controller.setState();
                  })),
          Row(
            children: <Widget>[
              SizedBox(width: 10),
              Expanded(
                child: _propertyList(filter),
              ),
              _operatorList(filter)
            ],
          ),
          Padding(
              child: _valueWidget(filter),
              padding: EdgeInsets.only(top: 0, bottom: 0, right: 10, left: 10))
        ],
      ),
    );
  }

  Widget _propertyList(EntityFilter filter) {
    if (_controller.properties.length == 0) return Container();
    return DropdownButtonHideUnderline(
        child: DropdownButton(
      onChanged: (v) {
        filter.property = v as Property;
        filter.propertyName = filter.property?.dbName;
        filter.value = null;
        setState(() {});
      },
      value: _controller.properties
          // ignore: unnecessary_cast
          .map<Property?>((p) => p as Property?)
          .firstWhere(
              (p) =>
                  p?.dbName == (filter.propertyName ?? filter.property?.dbName),
              orElse: () => null),
      items: _controller.properties
          .map((p) => DropdownMenuItem(child: TecfyText(p.nameAr), value: p))
          .toList(),
    ));
  }

  DropdownButton _operatorList(EntityFilter filter) {
    return DropdownButton(
      onChanged: (v) {
        filter.operatorId = v;
        setState(() {});
      },
      value: filter.operatorId,
      items: [
        DropdownMenuItem(child: TecfyText('='), value: 'E'),
        DropdownMenuItem(child: TecfyText('>='), value: 'GE'),
        DropdownMenuItem(child: TecfyText('<='), value: 'LE'),
        DropdownMenuItem(
            child: TecfyText(AppLocale.translate('Contain')), value: 'C'),
        DropdownMenuItem(
            child: TecfyText(AppLocale.translate('Not Contain')), value: 'NC'),
      ],
    );
  }

  Widget _valueWidget(EntityFilter filter) {
    if (filter.property == null)
      return Container();
    else
      return PorpertyComponent(
          property: filter.property!,
          entity: Entity.fromMap({}),
          onChange: (v) {
            setState(() => filter.value = v);
          },
          data: {filter.property?.dbName ?? '-': filter.value});
  }
}
